@@ -1,6 +1,6 @@
 r"""
 
-ImageJ
+imagej
 ######
 
 **Access ImageJ Filters**
@@ -11,8 +11,8 @@ ImageJ
    :template: mybase.rst
    :toctree: generated/
 
-    imagej.imagej_wrapper
-    imagej.imagej_plugin
+    filters.imagej.imagej_wrapper
+    filters.imagej.imagej_plugin
 
 """
 
